// MyTask.c

#include "includes.h"

#define	TASK_STK_SIZE	1024							// Stack size, in bytes
#define	N_TASKS			4								// Number of tasks

OS_STK	TaskStk[N_TASKS][TASK_STK_SIZE/sizeof(OS_STK)];	// Tasks stacks
// Prototypes

//GUNMAN_2005.05.03 : Tasks ---[
void	Task1(void * pParam);
void	Task2(void * pParam);
void	Task3(void * pParam);
void	Task4(void * pParam);
//]--

void	Sleep(INT16U uSec);
void	ClearScreen();
void	OnScreen(int x, int y, char * pszText, int nColor);

//GUNMAN_2005.05.03 : Events ---[
OS_EVENT *s_event[N_TASKS];
//]---

// Application entry point, invoked from Entry.obj
void main()
{
	int	Param1 = 0;	

	// Set the top of stack.
	__asm	MOV	ESP,0A0000h;

	// Display a banner.
	ClearScreen();	

	// Initialize uC/OS.
	OSCpuInit();
    OSInit();
		
	//GUNMAN_2005.05.03 : Keyboard ISR ---[
	// Initialize Semaphores
	s_event[0] = OSSemCreate(0);
	s_event[1] = OSSemCreate(0);
	s_event[2] = OSSemCreate(0);
	s_event[3] = OSSemCreate(0);
	
	
	// Create four tasks.
    OSTaskCreate(Task1, &Param1, &TaskStk[0][TASK_STK_SIZE], 0);
    OSTaskCreate(Task2, &Param1, &TaskStk[1][TASK_STK_SIZE], 1);
	OSTaskCreate(Task3, &Param1, &TaskStk[2][TASK_STK_SIZE], 2);
    OSTaskCreate(Task4, &Param1, &TaskStk[3][TASK_STK_SIZE], 3);	

	OnScreen(0,2, "Please Input Any Key '1' - '4'", 14); //
	//]---

	// Start multitasking.
    OSStart();	
	/* NEVER EXECUTED */
}

//GUNMAN_2005.05.03 : Tasks ---[
void Task1(void * pParam)
{
	int	iCount;
	char szStr[30];	
	unsigned char err;
	
	iCount = 0;	
		
	while(1)
	{					
		sprintf(szStr,"Task 1: %05d",iCount);
		OnScreen(10, 8, szStr, 10);
		iCount--;							
		if(iCount < 0){
			iCount = 50000;			
			OSSemPend(s_event[0], 0, &err);
		}		
    }	
}

void Task2(void * pParam)
{
	int	iCount;
	char szStr[30];	
	unsigned char err;
	
	iCount = 0;	
		
	while(1)
	{					
		sprintf(szStr,"Task 2: %05d",iCount);
		OnScreen(10, 9, szStr, 11);
		iCount--;							
		if(iCount < 0){
			iCount = 50000;
			OSSemPend(s_event[1], 0, &err);
		}
    }	
}

void Task3(void * pParam)
{
	int	iCount;
	char szStr[30];	
	unsigned char err;
	
	iCount = 0;	
		
	while(1)
	{					
		sprintf(szStr,"Task 3: %05d",iCount);
		OnScreen(10, 10, szStr, 12);
		iCount--;							
		if(iCount < 0){
			iCount = 50000;			
			OSSemPend(s_event[2], 0, &err);
		}
    }	
}

void Task4(void * pParam)
{
	int	iCount;
	char szStr[30];	
	unsigned char err;
	
	iCount = 0;	
		
	while(1)
	{					
		sprintf(szStr,"Task 4: %05d",iCount);
		OnScreen(10, 11, szStr, 13);
		iCount--;							
		if(iCount < 0){
			iCount = 50000;
			OSSemPend(s_event[3], 0, &err);
		}
    }	
}
//]---


// Sleeps for the specified delay (in seconds).
// Since the 8254 is not set in this implementation,
// sleeping one second means sleeping 18 clock ticks.

void Sleep(INT16U uSec)
{
	OSTimeDly((INT16U) (18 * uSec));
}

// Clears the screen.

void ClearScreen()
{
	int		x,y;
	char *	pScreen = (char *) 0xb8000;

	for (y = 0; y < 25; y++)
		for (x = 0; x < 80; x++)
		{
			*pScreen++ = ' ';
			*pScreen++ = 7;
		}
}

// Displays a string on the screen, assuming a 80x25 color adapter.

void OnScreen(int x, int y, char * pszText, int nColor)
{
	char *	pScreen = (char *) (0xb8000 + y * 160 + x * 2);

	while (*pszText)
	{
		*pScreen++ = *pszText++;
		*pScreen++ = nColor;
	}
}

//GUNMAN_2005.05.03 : Keyboard ISR ---[
void KeyBoardISRHandler()
{
	char szKey[5];
	int i;
	
	i=inportb(0x64);

	if(i & 0x01){	
		i=inportb(0x60);
		i--;

		if( i >= 1 && i <= 4 ){
			
			sprintf(szKey,"%02d",i);		
			OnScreen(0,0, szKey, 14);
			
			OSSemPost(s_event[i-1]);

		}
	}
}
//]---